
/***************************************************************************

                                 ChessV

                   COPYRIGHT (C) 2005 BY GREGORY STRONG

This file is part of ChessV.  ChessV is free software; you can redistribute
it and/or modify it under the terms of the GNU General Public License as 
published by the Free Software Foundation; either version 2 of the License, 
or (at your option) any later version.

ChessV is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
more details; the file 'COPYING' contains the License text, but if for
some reason you need a copy, please write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

****************************************************************************/


#include "StdAfx.h"
#include "Direction.h"
#include "MovementCapability.h"


// *** DEBUG MEMORY ALLOCATION *** //
#ifdef _DEBUG
#define new SAFE_NEW
#endif


Direction::Direction
	( int fileChange,
	  int rankChange ):
		fileDiff(fileChange),
		rankDiff(rankChange),
		opposite(NULL)
{
}

Direction::Direction
	( const Direction &other ):
		fileDiff(other.fileDiff),
		rankDiff(other.rankDiff),
		opposite(other.opposite),
		oppositeDirectionNumber(other.oppositeDirectionNumber)
{
}

Direction &Direction::operator=
	( const Direction &other )
{
	fileDiff = other.fileDiff;
	rankDiff = other.rankDiff;
	opposite = other.opposite;
	oppositeDirectionNumber = other.oppositeDirectionNumber;
	return( *this );
}

void Direction::SetOpposite
	( Direction &other )
{
	opposite = &other;
}

void Direction::SetOppositeDirectionNumber
	( int oppositeDirection )
{
	oppositeDirectionNumber = oppositeDirection;
}

bool Direction::operator==
	( const MovementCapability &other ) const
{
	return( fileDiff == other.GetFileDifference( 0 ) &&
		    rankDiff == other.GetRankDifference( 0 ) );
}
